==> react-app-logic-17.1.0.tgz: package/src/exchange/forms/usePriceInputs.ts <==
import { useCallback, useState } from 'react';
import BigNumber from 'bignumber.js';
import { DetailedFungibleAssetOnChain } from '@onomy/hub';
import { useDetailedBalance } from '@onomy/react-hub';
import { useFixedAmountFromDisplay } from '../hooks';

const BN_ZERO = new BigNumber(0);
const BN_ONE = new BigNumber(1);

const DEFAULT_STATE = {
  rate: ['0', '0'],
  price: '0',
  inversePrice: '0',
  amountLastChanged: 'base' as 'base' | 'quote',
  amountBaseFixed: '0',
  amountQuoteFixed: '0',
};

const MAX_RATE_DENOMINATOR = '18446744073709551614';

function calculateRate({
  price,
  baseAsset,
  quoteAsset,
}: {
  price: string;
  baseAsset: DetailedFungibleAssetOnChain;
  quoteAsset: DetailedFungibleAssetOnChain;
}) {
  let bnPrice = new BigNumber(price);
  if (bnPrice.isNaN()) bnPrice = BN_ZERO;

  const expShift = baseAsset.meta.decimalPlaces - quoteAsset.meta.decimalPlaces;
  let bnInversePrice = BN_ONE.dividedBy(bnPrice).shiftedBy(-1 * expShift);
  if (!bnInversePrice.isFinite()) bnInversePrice = BN_ZERO;

  bnPrice = bnPrice.shiftedBy(expShift);
  const rate = bnPrice.eq(0)
    ? ['0', '0']
    : (bnPrice.toFraction(MAX_RATE_DENOMINATOR).map(x => x.toFixed(0)) as [string, string]);

  while (BigNumber(rate[0]).gte(MAX_RATE_DENOMINATOR)) {
    // Ensure numerator is also less than MAX_RATE_DENOMINATOR
    rate[0] = rate[0].slice(0, rate[0].length - 1);
    rate[1] = rate[1].slice(0, rate[1].length - 1);
  }

  return {
    price,
    bnPrice,
    bnInversePrice,
    rate,
    inversePrice: bnInversePrice.toString(10),
  };
}

function calculatePrice({
  baseAmountFixed,
  quoteAmountFixed,
  baseAsset,
  quoteAsset,
}: {
  baseAmountFixed: string;
  quoteAmountFixed: string;
  baseAsset: DetailedFungibleAssetOnChain;
  quoteAsset: DetailedFungibleAssetOnChain;
}) {
  const bnQuote = new BigNumber(quoteAmountFixed).shiftedBy(
    -1 * (quoteAsset.meta.decimalPlaces ?? 18)
  );
  const bnBase = new BigNumber(baseAmountFixed).shiftedBy(
    -1 * (baseAsset.meta.decimalPlaces ?? 18)
  );
  const price = bnQuote.dividedBy(bnBase).toString(10);
  return calculateRate({ price, baseAsset, quoteAsset });
}

export function usePriceInputs({
  baseAsset,
  quoteAsset,
  updateOtherAsset = true,
}: {
  baseAsset: DetailedFungibleAssetOnChain;
  quoteAsset: DetailedFungibleAssetOnChain;
  updateOtherAsset?: boolean;
}) {
  const [{ rate, price, inversePrice, amountBaseFixed, amountQuoteFixed }, setState] =
    useState(DEFAULT_STATE);

  const priceFixed = useFixedAmountFromDisplay(price, quoteAsset);
  const [pricePerBase, pricePerBaseStatus] = useDetailedBalance(quoteAsset, priceFixed);
  const inversePriceFixed = useFixedAmountFromDisplay(inversePrice, baseAsset);
  const [inversePricePerBase, inversePricePerBaseStatus] = useDetailedBalance(
    baseAsset,
    inversePriceFixed
  );

  const setPrice = useCallback(
    (newPrice: string) => {
      setState(current => {
        const {
          bnPrice: calculatedBnPrice,
          inversePrice: calculatedInversePrice,
          rate: calculatedRate,
        } = calculateRate({
          price: newPrice,
          baseAsset,
          quoteAsset,
        });

        if (current.amountLastChanged === 'base') {
          const bnAmount = new BigNumber(current.amountBaseFixed);
          const calculatedQuote = bnAmount.multipliedBy(calculatedBnPrice);
          return {
            ...current,
            amountQuoteFixed:
              calculatedQuote.isNaN() || !calculatedQuote.isFinite()
                ? '0'
                : calculatedQuote.toFixed(0),
            rate: calculatedRate,
            price: newPrice,
            inversePrice: calculatedInversePrice,
          };
        }
        const bnAmount = new BigNumber(current.amountQuoteFixed);
        const calculatedQuote = bnAmount.dividedBy(calculatedBnPrice);
        return {
          ...current,
          amountBaseFixed:
            calculatedQuote.isNaN() || !calculatedQuote.isFinite()
              ? '0'
              : calculatedQuote.toFixed(0),
          rate: calculatedRate,
          price: newPrice,
          inversePrice: calculatedInversePrice,
        };
      });
    },
    // eslint-disable-next-line react-hooks/exhaustive-deps
    [baseAsset.meta.decimalPlaces, quoteAsset.meta.decimalPlaces]
  );

  const setAmountBaseFixed = useCallback(
    (newAmountFixed: string) => {
      setState(current => {
        if (updateOtherAsset) {
          const bnAmount = new BigNumber(newAmountFixed);
          const expShift = baseAsset.meta.decimalPlaces - quoteAsset.meta.decimalPlaces;
          const bnPrice = new BigNumber(current.price).shiftedBy(expShift);
          const calculatedQuote = bnAmount.multipliedBy(bnPrice);

          return {
            ...current,
            amountLastChanged: 'base',
            amountBaseFixed: newAmountFixed,
            amountQuoteFixed:
              calculatedQuote.isNaN() || !calculatedQuote.isFinite()
                ? '0'
                : calculatedQuote.toFixed(0),
          };
        }
        const {
          price: calculatedPrice,
          inversePrice: calculatedInversePrice,
          rate: calculatedRate,
        } = calculatePrice({
          baseAsset,
          quoteAsset,
          baseAmountFixed: newAmountFixed,
          quoteAmountFixed: current.amountQuoteFixed,
        });

        return {
          ...current,
          price: calculatedPrice,
          inversePrice: calculatedInversePrice,
          rate: calculatedRate,
          amountLastChanged: 'base',
          amountBaseFixed: newAmountFixed,
        };
      });
    },
    // eslint-disable-next-line react-hooks/exhaustive-deps
    [updateOtherAsset, baseAsset.meta.decimalPlaces, quoteAsset.meta.decimalPlaces]
  );

  const setAmountQuoteFixed = useCallback(
    (newAmountFixed: string) => {
      setState(current => {
        if (updateOtherAsset) {
          const bnAmount = new BigNumber(newAmountFixed);
          const expShift = baseAsset.meta.decimalPlaces - quoteAsset.meta.decimalPlaces;
          const bnPrice = new BigNumber(current.price).shiftedBy(expShift);
          const calculatedBase = bnAmount.dividedBy(bnPrice);

          return {
            ...current,
            amountLastChanged: 'quote',
            amountQuoteFixed: newAmountFixed,
            amountBaseFixed:
              calculatedBase.isNaN() || !calculatedBase.isFinite()
                ? '0'
                : calculatedBase.toFixed(0),
          };
        }
        const {
          price: calculatedPrice,
          inversePrice: calculatedInversePrice,
          rate: calculatedRate,
        } = calculatePrice({
          baseAsset,
          quoteAsset,
          quoteAmountFixed: newAmountFixed,
          baseAmountFixed: current.amountBaseFixed,
        });

        return {
          ...current,
          price: calculatedPrice,
          inversePrice: calculatedInversePrice,
          rate: calculatedRate,
          amountLastChanged: 'quote',
          amountQuoteFixed: newAmountFixed,
        };
      });
    },
    // eslint-disable-next-line react-hooks/exhaustive-deps
    [updateOtherAsset, baseAsset.meta.decimalPlaces, quoteAsset.meta.decimalPlaces]
  );

  const reset = useCallback(() => setState(DEFAULT_STATE), []);

  return {
    rate,
    price,
    inversePrice,
    amountBaseFixed,
    amountQuoteFixed,

    priceFixed,
    pricePerBase,
    pricePerBaseStatus,
    inversePriceFixed,
    inversePricePerBase,
    inversePricePerBaseStatus,

    setPrice,
    setAmountBaseFixed,
    setAmountQuoteFixed,

    reset,
  };
}

==> react-app-logic-18.0.3.tgz: package/src/exchange/forms/usePriceInputs.ts <==
import { useCallback, useState } from 'react';
import BigNumber from 'bignumber.js';
import { DetailedFungibleAssetOnChain } from '@onomy/hub';
import { useDetailedBalance } from '@onomy/react-hub';
import { useFixedAmountFromDisplay } from '../hooks';

const BN_ZERO = new BigNumber(0);
const BN_ONE = new BigNumber(1);

const DEFAULT_STATE = {
  rate: ['0', '0'],
  price: '0',
  inversePrice: '0',
  amountLastChanged: 'base' as 'base' | 'quote',
  amountBaseFixed: '0',
  amountQuoteFixed: '0',
};

const MAX_RATE_DENOMINATOR = '18446744073709551614';

function calculateRate({
  price,
  baseAsset,
  quoteAsset,
}: {
  price: string;
  baseAsset: DetailedFungibleAssetOnChain;
  quoteAsset: DetailedFungibleAssetOnChain;
}) {
  let bnPrice = new BigNumber(price);
  if (bnPrice.isNaN()) bnPrice = BN_ZERO;

  const expShift = quoteAsset.meta.decimalPlaces - baseAsset.meta.decimalPlaces;
  let bnInversePrice = BN_ONE.dividedBy(bnPrice).shiftedBy(-1 * expShift);
  if (!bnInversePrice.isFinite()) bnInversePrice = BN_ZERO;

  bnPrice = bnPrice.shiftedBy(expShift);
  const rate = bnPrice.eq(0)
    ? ['0', '0']
    : (bnPrice.toFraction(MAX_RATE_DENOMINATOR).map(x => x.toFixed(0)) as [string, string]);

  while (BigNumber(rate[0]).gte(MAX_RATE_DENOMINATOR)) {
    // Ensure numerator is also less than MAX_RATE_DENOMINATOR
    rate[0] = rate[0].slice(0, rate[0].length - 1);
    rate[1] = rate[1].slice(0, rate[1].length - 1);
  }

  return {
    price,
    bnPrice,
    bnInversePrice,
    rate,
    inversePrice: bnInversePrice.toString(10),
  };
}

function calculatePrice({
  baseAmountFixed,
  quoteAmountFixed,
  baseAsset,
  quoteAsset,
}: {
  baseAmountFixed: string;
  quoteAmountFixed: string;
  baseAsset: DetailedFungibleAssetOnChain;
  quoteAsset: DetailedFungibleAssetOnChain;
}) {
  const bnQuote = new BigNumber(quoteAmountFixed).shiftedBy(
    -1 * (quoteAsset.meta.decimalPlaces ?? 18)
  );
  const bnBase = new BigNumber(baseAmountFixed).shiftedBy(
    -1 * (baseAsset.meta.decimalPlaces ?? 18)
  );
  const price = bnQuote.dividedBy(bnBase).toString(10);

  return calculateRate({ price, baseAsset, quoteAsset });
}

export function usePriceInputs({
  baseAsset,
  quoteAsset,
  updateOtherAsset = true,
}: {
  baseAsset: DetailedFungibleAssetOnChain;
  quoteAsset: DetailedFungibleAssetOnChain;
  updateOtherAsset?: boolean;
}) {
  const [{ rate, price, inversePrice, amountBaseFixed, amountQuoteFixed }, setState] =
    useState(DEFAULT_STATE);

  const priceFixed = useFixedAmountFromDisplay(price, quoteAsset);
  const [pricePerBase, pricePerBaseStatus] = useDetailedBalance(quoteAsset, priceFixed);
  const inversePriceFixed = useFixedAmountFromDisplay(inversePrice, baseAsset);
  const [inversePricePerBase, inversePricePerBaseStatus] = useDetailedBalance(
    baseAsset,
    inversePriceFixed
  );

  const setPrice = useCallback(
    (newPrice: string) => {
      setState(current => {
        const {
          bnPrice: calculatedBnPrice,
          inversePrice: calculatedInversePrice,
          rate: calculatedRate,
        } = calculateRate({
          price: newPrice,
          baseAsset,
          quoteAsset,
        });

        if (current.amountLastChanged === 'base') {
          const bnAmount = new BigNumber(current.amountBaseFixed);
          const calculatedQuote = bnAmount.multipliedBy(calculatedBnPrice);
          return {
            ...current,
            amountQuoteFixed:
              calculatedQuote.isNaN() || !calculatedQuote.isFinite()
                ? '0'
                : calculatedQuote.toFixed(0),
            rate: calculatedRate,
            price: newPrice,
            inversePrice: calculatedInversePrice,
          };
        }
        const bnAmount = new BigNumber(current.amountQuoteFixed);
        const calculatedQuote = bnAmount.dividedBy(calculatedBnPrice);
        return {
          ...current,
          amountBaseFixed:
            calculatedQuote.isNaN() || !calculatedQuote.isFinite()
              ? '0'
              : calculatedQuote.toFixed(0),
          rate: calculatedRate,
          price: newPrice,
          inversePrice: calculatedInversePrice,
        };
      });
    },
    // eslint-disable-next-line react-hooks/exhaustive-deps
    [baseAsset.meta.decimalPlaces, quoteAsset.meta.decimalPlaces]
  );

  const setAmountBaseFixed = useCallback(
    (newAmountFixed: string) => {
      setState(current => {
        if (updateOtherAsset) {
          const bnAmount = new BigNumber(newAmountFixed);
          const expShift = quoteAsset.meta.decimalPlaces - baseAsset.meta.decimalPlaces;
          const bnPrice = new BigNumber(current.price).shiftedBy(expShift);
          const calculatedQuote = bnAmount.multipliedBy(bnPrice);

          return {
            ...current,
            amountLastChanged: 'base',
            amountBaseFixed: newAmountFixed,
            amountQuoteFixed:
              calculatedQuote.isNaN() || !calculatedQuote.isFinite()
                ? '0'
                : calculatedQuote.toFixed(0),
          };
        }
        const {
          price: calculatedPrice,
          inversePrice: calculatedInversePrice,
          rate: calculatedRate,
        } = calculatePrice({
          baseAsset,
          quoteAsset,
          baseAmountFixed: newAmountFixed,
          quoteAmountFixed: current.amountQuoteFixed,
        });

        return {
          ...current,
          price: calculatedPrice,
          inversePrice: calculatedInversePrice,
          rate: calculatedRate,
          amountLastChanged: 'base',
          amountBaseFixed: newAmountFixed,
        };
      });
    },
    // eslint-disable-next-line react-hooks/exhaustive-deps
    [updateOtherAsset, baseAsset.meta.decimalPlaces, quoteAsset.meta.decimalPlaces]
  );

  const setAmountQuoteFixed = useCallback(
    (newAmountFixed: string) => {
      setState(current => {
        if (updateOtherAsset) {
          const bnAmount = new BigNumber(newAmountFixed);
          const expShift = quoteAsset.meta.decimalPlaces - baseAsset.meta.decimalPlaces;
          const bnPrice = new BigNumber(current.price).shiftedBy(expShift);
          const calculatedBase = bnAmount.dividedBy(bnPrice);

          return {
            ...current,
            amountLastChanged: 'quote',
            amountQuoteFixed: newAmountFixed,
            amountBaseFixed:
              calculatedBase.isNaN() || !calculatedBase.isFinite()
                ? '0'
                : calculatedBase.toFixed(0),
          };
        }
        const {
          price: calculatedPrice,
          inversePrice: calculatedInversePrice,
          rate: calculatedRate,
        } = calculatePrice({
          baseAsset,
          quoteAsset,
          quoteAmountFixed: newAmountFixed,
          baseAmountFixed: current.amountBaseFixed,
        });

        return {
          ...current,
          price: calculatedPrice,
          inversePrice: calculatedInversePrice,
          rate: calculatedRate,
          amountLastChanged: 'quote',
          amountQuoteFixed: newAmountFixed,
        };
      });
    },
    // eslint-disable-next-line react-hooks/exhaustive-deps
    [updateOtherAsset, baseAsset.meta.decimalPlaces, quoteAsset.meta.decimalPlaces]
  );

  const reset = useCallback(() => setState(DEFAULT_STATE), []);

  return {
    rate,
    price,
    inversePrice,
    amountBaseFixed,
    amountQuoteFixed,

    priceFixed,
    pricePerBase,
    pricePerBaseStatus,
    inversePriceFixed,
    inversePricePerBase,
    inversePricePerBaseStatus,

    setPrice,
    setAmountBaseFixed,
    setAmountQuoteFixed,

    reset,
  };
}
